/***************************************************************************
                          common.cpp  -  description
                             -------------------
    begin                : Thu Apr 11 2002
    copyright            : (C) 2002 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


// This file contains common functions, that may be used throughout the library

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "constants.h"
#include "structs.h"
#include "externs.h"
#include "common.h"



void giveup (char *string1, char *string2)
// to add: variable nb of parameters, as in scanf, printf
{
    char temp[100];
    sprintf (temp, "%s %s", string1, string2);
    perror (temp);
    exit(1);
}


void create_random_sequence (int length, char *sequence)
// function to create uniformly random sequences - for demonstration purposes
{
    int rnumber, i;
    char base;
    for (i=0; i< length; i++)
    {
        rnumber = rand() % 100;
        if (rnumber >=0 && rnumber < 25)
            base = 'A';
        else if (rnumber >= 25 && rnumber < 50 )
            base = 'C';
        else if (rnumber >= 50 && rnumber < 75)
            base = 'G';
        else
            base = 'U';
        sequence[i] = base;
    }
    sequence[i] = '\0';
}


void remove_space (char *structure)
// PRE: none
// POST: remove the space(s) from structure, if any; modifies structure
{
    char str2[MAXSLEN];
    int len,i,j;
    len = strlen(structure);
    j = 0;
    for (i=0; i < len; i++)
    {
        if (structure[i] != ' ')
            str2[j++] = structure[i];
    }
    str2[j] = '\0';
    strcpy (structure, str2);    
}


void empty_string (char * str)
// PRE:  str is a string
// POST: Put '\0' at all positions
{
    int i;
    int len;
    len = strlen(str);
    for (i=0; i< len; i++)
        str[i] = '\0';
}

int can_pair (int base1, int base2)
// PRE:  base1 and base2 are nucleotides over the alphabet {A, C, G, T, U}
// POST: return 1 if they can pair, 0 otherwise
{
    switch (base1)
    {
        case A:
            switch (base2)
            {
                case U: return 1;
                default: return 0;
            }
        case C:
            switch (base2)
            {
                case G: return 1;
                default: return 0;
            }                        
        case G:
            switch (base2)
            {
                case C: case U: return 1;
                default: return 0;
            }
        default:
            switch (base2)
            {
                case A: case G: return 1;
                default: return 0;
            }               
    }  
}


int nuc_to_int (char nucleotide)
// PRE:  nucleotide is 'A', 'C', 'G' or 'T'
// POST: Return 0 for A, 1 for C, 2 for G, 3 for T
{
    switch(nucleotide)
    {
        case 'a':
        case 'A': return A;
        case 'c':
        case 'C': return C;
        case 'g':
        case 'G': return G;
        default : return U;
    }
}


/*
// by Cho MinSik
 int nuc_to_int (char nucleotide)
// PRE:  nucleotide is 'A', 'C', 'G' or 'T'
// POST: Return 0 for A, 1 for C, 2 for G, 3 for T

{
    static bool bInitialized    =   false;
    static int  iNucleotide['z'];

    if(!bInitialized)
    {
        bInitialized    =   true;
        iNucleotide['a']    =   A;
        iNucleotide['A']    =   A;
        iNucleotide['c']    =   C;
        iNucleotide['C']    =   C;
        iNucleotide['g']    =   G;
        iNucleotide['G']    =   G;
        iNucleotide['t']    =   U;
        iNucleotide['T']    =   U;
        iNucleotide['u']    =   U;
        iNucleotide['U']    =   U;
    }
    return iNucleotide[nucleotide];
}
*/

char int_to_nuc (int inuc)
{
    switch(inuc)
    {
        case A: return 'A';
        case C: return 'C';
        case G: return 'G';
        default: return 'U';
    }
}


int is_nucleotide (char base)
// PRE:  base is a character 
// POST: return true if base is a nucleotide (A, C, G or T)
//     return false otherwise
{
    return (base == 'A' || base == 'C' || base == 'G' || base == 'T' || base == 'U' ||
            base == 'a' || base == 'c' || base == 'g' || base == 't' || base == 'u');
}


void check_sequence (char *sequence)
// check sequence for length and alphabet
{
    int length;
    int maxlen;
    int i;
    length = strlen (sequence);
    maxlen = MAXSLEN;
    if (length == 0)
    {
        printf ("Empty sequence\n");
        exit (1);
    }
    if (length > MAXSLEN)
    {
        printf ("Sequence too long; maximum allowed: %d\n", maxlen);
        exit (1);
    }
    for (i=0; i<length; i++)
    {
        if (!is_nucleotide (sequence[i]))
        {
            printf ("Sequence not valid: %c found\n", sequence[i]);
            printf ("Only acgtuACGTU are allowed\n");
            exit (1);
        }
    }
}


int penalty_by_size (int size, char type)
// PRE:  size is the size of the loop
//       type is HAIRP or INTER or BULGE
// POST: return the penalty by size of the loop
{
    int penalty30, penalty;

    // the penalties for size <= MAXLOOP should be read from the file "loop"
    if (size <= MAXLOOP)
    {
        if (type == 'H')
            return hairpin_penalty_by_size[size];
        if (type == 'I')
            return internal_penalty_by_size[size];
        return bulge_penalty_by_size[size];
    }

    // size > MAXLOOP
    if (type == 'H')
        penalty30 = hairpin_penalty_by_size[MAXLOOP];
    else if (type == 'I')
        penalty30 = internal_penalty_by_size[MAXLOOP];
    else
        penalty30 = bulge_penalty_by_size[MAXLOOP];

    // by Cho MinSik
    static  double LogValue[INF];
    if(LogValue[size]==0)
        LogValue[size]  =   log(((double)size)/30);

    penalty = (int) (penalty30 + misc.param_greater30 * LogValue[size]);

    return penalty;
}


int penalty_by_size_enthalpy (int size, char type)
// PRE:  size is the size of the loop
//       type is HAIRP or INTER or BULGE
// POST: return the penalty by size of the loop
{
    int penalty30, penalty;

    // the penalties for size <= MAXLOOP should be read from the file "loop"
    if (size <= MAXLOOP)
    {
        if (type == 'H')
            return enthalpy_hairpin_penalty_by_size[size];
        if (type == 'I')
            return enthalpy_internal_penalty_by_size[size];
        return enthalpy_bulge_penalty_by_size[size];
    }

    // size > MAXLOOP
    if (type == 'H')
        penalty30 = enthalpy_hairpin_penalty_by_size[MAXLOOP];
    else if (type == 'I')
        penalty30 = enthalpy_internal_penalty_by_size[MAXLOOP];
    else
        penalty30 = enthalpy_bulge_penalty_by_size[MAXLOOP];

    penalty = (int) (penalty30 + enthalpy_misc.param_greater30 * log(((double)size)/30));

    return penalty;
}


void substr (char *source, int begin, int end, char *dest)
// PRE:  begin and end are smaller than strlen(source)
// POST: Put in dest what is in source between position begin and position end
{
    int i;
    for (i = 0; i <= end - begin; i++)
        dest[i] = source[i+begin];
    dest[i] = '\0';
}


char complement (char base)
// PRE:  Base is an RNA nucleotide
// POST: Return the complement of base
{
  switch (base)
  {
    case 'a': case 'A': return 'U';
    case 'c': case 'C': return 'G';
    case 'g': case 'G': return 'C';
    default: return 'A';
  }
}


void reverse_complement_of_seq (const char *seq, char *complem)
// PRE:  seq is a sequence
// POST: complement and reverse sequence and put the result into compl
{
    int seq_len, i;
    seq_len = strlen(seq);
    for (i=0; i< seq_len; i++)
    {
        complem[i] = complement (seq[seq_len-i-1]);
    }
    complem[seq_len] = '\0';
}




void insert_space (char *structure, int place)
// PRE:  None
// POST: insert a space at the specified place, in structure
{
    int i;
    int length;
    char str[MAXSLEN];
    length = strlen (structure);
    for (i=0; i <= place; i++)
        str[i] = structure[i];
    str[i] = ' ';
    for (i=place+1; i < length; i++)
        str[i+1] = structure[i];
    str[i+1]= '\0';
    strcpy (structure, str);
}


void init (stack_ds *st)
// PRE:  None
// POST: Initialize the stack st
{
        st->top = 0;
}

void push (stack_ds *st, int el)
// PRE:  st is a valid stack
// POST: Push an element to the stack
{
        st->elem[st->top++] = el;
}

int pop (stack_ds *st)
// PRE:  st is a valid stack, that is not empty
// POST: pop an element from the stack and return it
{
    if (st->top <= 0)
    {
        fprintf (stderr, "The given structure is not valid: more right parentheses than left parentheses\n");
        exit (1);
    }
        return st->elem[--st->top];
}



void detect_original_pairs(char *structure, int *p_table)
// PRE:  structure contains the desired structure
// POST: pairs will contain the index of each base pair
//               or -1 if it does not pair
{
        int i, j, struct_len;
        stack_ds st;
        init (&st);
        remove_space (structure);
        struct_len = strlen (structure);
        for (i=0; i < struct_len; i++)
        {
                if (structure[i] == '.' || structure[i] == ' ')
                        p_table[i] = -1;
                else if (structure[i] == '(')
                        push (&st, i);
                else if (structure[i] == ')')
                {
                        j = pop (&st);
                        p_table[i] = j;
                        p_table[j] = i;
                }
        }
        if (st.top != 0)
        {
            fprintf (stderr, "The given structure is not valid: %d more left parentheses than right parentheses\n", st.top);
            exit (1);
        }
}



void detect_structure_features (char *structure, str_features *f)
// PRE:  None
// POST: The variable f is filled with structure features, i.e. what type of elementary structure
//       this base is closing (such as stacked pair, hairpin loop etc.)         
{
    int num_branches, i, j;
    int p_table[MAXSLEN];
    int bri[MAX_BRANCHES];
    int nb_nucleotides;

    nb_nucleotides = strlen(structure);
    detect_original_pairs (structure, p_table);
    for (i=0; i < nb_nucleotides; i++)
    {
        if (p_table[i] > i)
        {
            f[i].pair = p_table[i];
            f[p_table[i]].pair = i;
            // check if it is stacked pair
            if (p_table[i+1] == p_table[i]-1 && p_table[i+1] > i+1)
            {
                f[i].type = STACK;
                f[p_table[i]].type = STACK;
                continue;
            }
            // check if it is hairpin, internal loop or multi-loop
            num_branches = 0;
            for (j=i+1; j < p_table[i]; j++)
            {
                if (p_table[j] > j)
                {
                    bri[num_branches] = j;
                    num_branches++;
                    j = p_table[j];
                }
            }
            if (num_branches == 0)  // hairpin
            {
                f[i].type = HAIRP;
                f[p_table[i]].type = HAIRP;
            }
            else if (num_branches == 1) // internal loop
            {
                f[i].type = INTER;
                f[p_table[i]].type = INTER;
                f[i].num_branches = 1;
                f[i].bri[0] = bri[0];
            }
            else    // multi loop
            {
                f[i].type = MULTI;
                f[p_table[i]].type = MULTI;
                f[i].num_branches = num_branches;
                for (j=0; j < num_branches; j++)
                    f[i].bri[j] = bri[j];
            }
        }
    }
    /*
    for (i=0; i < nb_nucleotides; i++)
    {
        if (f[i].pair > i)
        {
            printf ("%d - pair: %d type: %c ", i, f[i].pair, f[i].type);
            if (f[i].type == INTER)
                printf ("(%d,%d)", f[i].bri[0], f[f[i].bri[0]].pair);
            else if (f[i].type == MULTI)
                for (j=0; j < f[i].num_branches; j++)
                    printf ("(%d,%d) ", f[i].bri[j], f[f[i].bri[j]].pair);
            printf ("\n");
        }
    }
    */
}


int complementary_bases (char b1, char b2)
// returns 1 if b1 and b2 are complementary bases
{
    int base1, base2;
    base1 = nuc_to_int (b1);
    base2 = nuc_to_int (b2);
    switch (base1)
    {
        case A:
            switch (base2)
            {
                case U: return 1;
                default: return 0;
            }
        case C:
            switch (base2)
            {
                case G: return 1;
                default: return 0;
            }                        
        case G:
            switch (base2)
            {
                case C: return 1;
                default: return 0;
            }
        default:
            switch (base2)
            {
                case A: return 1;
                default: return 0;
            }               
    }      
}

int self_complementary (char *sequence)
// return 1 if this sequence is self-complementary
// self_complementary means the first half is the reverse complement of the second half
// if length (sequence) is an odd number, which one is true?
//    - is not self-comp
//    - the middle base does not matter
{
    int i, len;
    len = strlen (sequence);
    // if off, return 0;
    if ((len/2)*2 != len)
        return 0;
    for (i=0; i < len/2; i++)
    {
        if (!complementary_bases (sequence[i], sequence[len-i-1]))
            return 0;
    }
    return 1;
}
