/***************************************************************************
                          s_energy_matrix.h  -  description
                             -------------------
    begin                : Fri Apr 12 2002
    copyright            : (C) 2002 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ENERGY_MATRIX_H
#define ENERGY_MATRIX_H

//#include "s_stacked_pair.h"
#include "s_hairpin_loop.h"
#include "s_internal_loop.h"
#include "s_multi_loop.h"



class s_energy_matrix
{
    public:

        friend class s_stacked_pair;
        friend class s_internal_loop;
        friend class s_multi_loop;

        s_energy_matrix (int *seq, int length);
        // The constructor

        ~s_energy_matrix ();
        // The destructor

        void set_loops (s_hairpin_loop *H, s_stacked_pair *S,
                        s_internal_loop *VBI, s_multi_loop *VM)
        // Set the local loops to the given values
        {
            this->H = H;
            this->S = S;
            this->VBI = VBI;
            this->VM = VM;        
        }

        void compute_energy (int i, int j);
        // compute the V(i,j) value

        free_energy_node* get_node (int i, int j) { int ij = index[i]+j-i; return &nodes[ij]; }
        // return the node at (i,j)
        
        int get_energy (int i, int j) { int ij = index[i]+j-i; return nodes[ij].energy; }
        // return the value at V(i,j)
        
        char get_type (int i, int j) { int ij = index[i]+j-i; return nodes[ij].type; }
        // return the type at V(i,j)
        

    private:
        s_hairpin_loop *H;
        s_stacked_pair *S;
        s_internal_loop *VBI;
        s_multi_loop *VM;

        int *sequence;             // the entire sequence for which we compute the energy
        int seqlen;                 // sequence length
        int *index;
        free_energy_node *nodes;  // hairpin nodes in a 1D array
};



#endif
