/***************************************************************************
                          s_min_folding.cpp  -  description
                             -------------------
    begin                : Thu Apr 11 2002
    copyright            : (C) 2002 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Supposing we have the energy matrixes constructed, we compute the MFE


#ifndef S_MIN_FOLDING_H
#define S_MIN_FOLDING_H


#include <stdio.h>
#include <string.h>
#include "structs.h"
#include "s_energy_matrix.h"
#include "s_multi_loop.h"


class s_min_folding
// Keeps all the valid RNA structure on a stack.
// 
{
    public:
  
        // The constructors
        s_min_folding (char *seq);

        // The destructor/
        ~s_min_folding ();
        double simfold ();
        void return_structure (char *structure) { strcpy (structure, this->structure); }        

    private:
        char* structure;
        s_hairpin_loop *H;
        s_stacked_pair *S;
        s_internal_loop *VBI;
        s_multi_loop *VM;
        s_energy_matrix *V;  
        int* int_sequence;

        minimum_fold *f;// the minimum folding; 
        int *W;
        int nb_nucleotides;
        char* sequence;
        seq_interval *stack_interval;

        void allocate_space();
        double fold_sequence ();
        void insert_node (int i, int j, char type);
        
        void backtrack (seq_interval *cur_interval);
        // PRE:  All matrixes V, VM, WM and W have been filled
        // POST: Discover the MFE path

        int compute_W_br2 (int j);
        void compute_W (int j);
        void print_result ();
};

#endif //S_MIN_FOLDING_H
